﻿-----------------
-- *** Config ***
-----------------

local showMainZoneText = true	-- Show main zone name
local showSubZoneText = true	-- Show subzone name
local showCoords = true			-- Show coordinates
local exactCoords = false 		-- Show one digit more

-----------------
-- *** Locale ***
-----------------

local L = GetLocale() == "deDE" and {
	zone = "Zone",
	subzone = "Unterzone",
	coordinates = "Koordinaten",
	leftClick = '|cff00ff00Links-klicken|r, um die Weltkarte aufzurufen.',
	shiftClick = "|cff00ff00Shift-klicken|r, um den Zonentext und die Koordinaten\nin die offene Chateditbox einzufügen.",
} or {
	zone = "Zone",
	subzone = "Subzone",
	coordinates = "Coordinates",
	leftClick = '|cff00ff00Left-Click|r toggles the worldmap.',
	shiftClick = "|cff00ff00Shift + Click|r inserts zone text and coords\ninto the open chat edit box.",
}

-----------------
-- *** Locals ***
-----------------

local text_zone = ""
local text_color = "ffff00"
local inInstanceStatus = false
local f = CreateFrame('Frame')
local elapsed = 0
local badinstanceType = {
	["arena"] = true,
	["party"] = true, -- No Outland dungeon maps :(
	["raid"] = true, -- No Outland raid maps :(
}
local coordsFormat = (showMainZoneText or showSubZoneText) and " (%.0f, %.0f)" or "%.0f, %.0f"
if exactCoords then 
	coordsFormat = coordsFormat:gsub('%%%.0f', '%%%.1f')
end

-------------------
-- *** LDB Init ***
-------------------

local obj = LibStub("LibDataBroker-1.1"):NewDataObject("Broker_Whereabout", {
	type = "data source",
	icon = "Interface\\WorldMap\\WorldMap-Icon",
	text = "",
})

------------------
-- *** Methods ***
------------------

local function GetCoords()
	if inInstanceStatus or (not showCoords) then
		return ""
	end
	local x, y = GetPlayerMapPosition("player")
	return string.format(coordsFormat, x * 100, y * 100)
end

local function UpdateZone()
	local pvpType = GetZonePVPInfo()
	if pvpType == "friendly" then
		text_color = "00ff00"
	elseif pvpType == "sanctuary" then
		text_color = "0FC5FF"
	elseif pvpType == "hostile" then
		text_color = "f03a00"
	elseif pvpType == "combat" then
		text_color = "f03a00"
	else
		text_color = "ffff00"
	end
	
	if showMainZoneText then
		local zoneText = GetRealZoneText() or ""
		local subZoneText = GetSubZoneText()
		if showSubZoneText and (zoneText ~= subZoneText) and (subZoneText ~= "") then
			text_zone = format("%s: %s", zoneText, subZoneText)
		else
			text_zone = zoneText
		end
	else
		text_zone = GetMinimapZoneText()
	end
end

local function UpdateLDBText()
	obj.text = format("|cff%s%s%s|r", text_color, text_zone, GetCoords())
end

local function CheckUpdate()
	local inInstance, instanceType = IsInInstance()
	if inInstance and badinstanceType[instanceType] then
		inInstanceStatus = true
	else
		inInstanceStatus = false
	end

	if not showCoords then
		return
	end

	local updating = f:GetScript('OnUpdate')
	if inInstanceStatus then
		if updating then
			f:SetScript('OnUpdate', nil)
		end
		return
	elseif updating then
		return
	end
	
	elapsed = 0
	f:SetScript('OnUpdate', function(self, el) -- XXX check for update probs while zoning
		elapsed = elapsed + el
		if (elapsed >= .25) then
			UpdateLDBText()
			elapsed = 0
		end
	end)
end

--------------------
-- *** LDB stuff ***
--------------------

obj.OnClick = function(_, button)
	if IsModifierKeyDown() then
		local subZoneText = GetSubZoneText()
		local zoneText = GetRealZoneText() or ""
		local x, y = GetPlayerMapPosition("player")
		local coords = x~=0 and y~=0 and format(" (%.0f, %.0f)", x * 100, y * 100) or ""
		local text = format("%s%s%s", zoneText, (zoneText ~= subZoneText) and (subZoneText ~= "") and format(": %s", subZoneText) or "", coords)
		if ChatEdit_GetActiveWindow() then
			ChatEdit_InsertLink(text)
		end
		--[[if ChatFrame1EditBox:IsVisible() then
			ChatFrame1EditBox:Insert(text)
		end--]]
	elseif button == "LeftButton" then
		ToggleFrame(WorldMapFrame)
	end
end

obj.OnTooltipShow = function(tooltip)
	if not tooltip or not tooltip.AddLine then 
		return
	end
	
	local subZoneText = GetSubZoneText()
	local zoneText = GetRealZoneText() or ""
	local x, y = GetPlayerMapPosition("player")
	local coords = x~=0 and y~=0 and format("%.0f, %.0f", x * 100, y * 100) or ""
	
	tooltip:AddLine("|cffff8800Broker: Whereabout|r")
	tooltip:AddLine(" ")
	
	tooltip:AddDoubleLine(L.zone, format("|cff%s%s|r",text_color,zoneText), 1, 1, 0, 1, 1, 1)
	if (zoneText ~= subZoneText) and (subZoneText ~= "") then
		tooltip:AddDoubleLine(L.subzone, format("|cff%s%s|r",text_color,subZoneText), 1, 1, 0, 1, 1, 1)
	end
	if coords then
		tooltip:AddDoubleLine(L.coordinates, format("|cff%s%s|r",text_color,coords), 1, 1, 0, 1, 1, 1)
	end
	
	tooltip:AddLine(" ")
	tooltip:AddLine(L.leftClick,1,1,0)
	tooltip:AddLine(L.shiftClick,1,1,0)
end

----------------------
-- *** Registering ***
----------------------

f:RegisterEvent("PLAYER_ENTERING_WORLD")
if showMainZoneText or showSubZoneText then
	f:RegisterEvent("ZONE_CHANGED")
	f:RegisterEvent("ZONE_CHANGED_INDOORS")
	f:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	f:RegisterEvent("WORLD_MAP_UPDATE")
end

f:SetScript('OnEvent', function(self, event, ...)
	if event == "PLAYER_ENTERING_WORLD" then
		CheckUpdate()
	end
	if showMainZoneText or showSubZoneText then 
		UpdateZone()
	end
	UpdateLDBText()
end)

